// Container module.
module SoS_Explorer.Objective

    // Constrain given architecture via chromosome fixing.
    let FixChromosome(characteristics : float[,], capabilities : bool[,],
                      feasibleInterfaces : bool[,], directedInterfaces : bool,
                      architecture : bool[]) =

        // Number of elements
        let numSystems         = Array2D.length1 characteristics
        let numCharacteristics = Array2D.length2 characteristics
        let numCapabilities    = Array2D.length2 capabilities

        // System indices
        let sys_fighter1 = 0
        let sys_fighter2 = 1
        let sys_fighter3 = 2
        let sys_RPA1     = 3
        let sys_RPA2     = 4
        let sys_RPA3     = 5
        let sys_RPA4     = 6
        let sys_U2       = 7
        let sys_DSP      = 8
        let sys_ftrSAR1  = 9
        let sys_ftrSAR2  = 10
        let sys_ftrSAR3  = 11
        let sys_JSTARS   = 12
        let sys_ThExp1   = 13
        let sys_ThExp2   = 14
        let sys_ConUS    = 15
        let sys_CmdCont1 = 16
        let sys_CmdCont2 = 17
        let sys_LOS1     = 18
        let sys_LOS2     = 19
        let sys_BLOS1    = 20
        let sys_BLOS2    = 21

        // Characteristic indices
        let char_IFDevCost = 0
        let char_OpsCost   = 1
        let char_Perf      = 2
        let char_DevTime   = 3

        // Capability indices
        let cap_EOIR    = 0
        let cap_SAR     = 1
        let cap_Exploit = 2
        let cap_C2      = 3
        let cap_Comm    = 4

        // Define hasSystem() function
        // Returns whether the given system is specified in the chromosome
        let hasSystem i =
            // Return whether system is specified
            architecture.[i]

        // Define hasInterface() function
        // Returns whether the given interface is specified in the chromosome
        let hasInterface i j =
            // If same system
            if i = j then
                false
            else
                // Index of interface in chromosome
                let jj = if j < i then j + 1 else j
                let k = 
                    if directedInterfaces then
                        (numSystems - 1) * (i + 1) + jj
                    else
                        let m = (min i j) + 1
                        m * numSystems - m * (m - 1) / 2 + (abs(i - j)) - 1
                // Return whether interface is specified
                architecture.[k]

        // Define setSystem() function
        // Sets the system in the chromosome to the specified value
        let setSystem i value =
            Array.set architecture i value |> ignore

        // Define setInterface() function
        // Sets the interface in the chromosome to the specified value
        let setInterface i j value =
            // If not same system
            if i <> j then
                // Index of interface in chromosome
                let jj = if j < i then j + 1 else j
                let k = 
                    if directedInterfaces then
                        (numSystems - 1) * (i + 1) + jj
                    else
                        let m = (min i j) + 1
                        m * numSystems - m * (m - 1) / 2 + (abs(i - j)) - 1
                Array.set architecture k value |> ignore

        (****************************************************)
        (* Replace below with the actual constraints!       *)
        (****************************************************)
        (* This is just an example of using the interfaces! *)
        (****************************************************)

        // Identify and add missing capabilities
        for i = 0 to numCapabilities - 1 do
            let mutable j = 0
            let mutable k = -1
            let mutable hasCapability = false
            while (not hasCapability) && (j < numSystems) do
                if capabilities.[j, i] then
                    if hasSystem j then
                        hasCapability <- true
                    else
                        k <- j
                j <- j + 1
            if (not hasCapability) && (k > -1) then
                setSystem k true

        // Remove infeasible interfaces from architecture
        for i = 0 to numSystems - 1 do
            for j = 0 to numSystems - 1 do
                // If different systems
                if i <> j then
                    // If interface is specified
                    if hasInterface i j then
                        // If infeasible, remove
                        if not (hasSystem i && hasSystem j && feasibleInterfaces.[i, j]) then
                            setInterface i j false
