// Container module.
module SoS_Explorer.Objective

    // Calculate the objective's value for a given architecture.
    let Objective_Flexibility(characteristics : float[,], capabilities : bool[,],
                              feasibleInterfaces : bool[,],directedInterfaces : bool,
                              negotiatedTime : float, negotiatedCost : float,
                              architecture : bool[]) =

        // Bounds
        let lowerBound =   0.0
        let upperBound = 100.0
        let bias       =   0.0

        // Number of elements
        let numSystems         = Array2D.length1 characteristics
        let numCharacteristics = Array2D.length2 characteristics
        let numCapabilities    = Array2D.length2 capabilities

        // System indices
        let sys_fighter1 = 0
        let sys_fighter2 = 1
        let sys_fighter3 = 2
        let sys_RPA1     = 3
        let sys_RPA2     = 4
        let sys_RPA3     = 5
        let sys_RPA4     = 6
        let sys_U2       = 7
        let sys_DSP      = 8
        let sys_ftrSAR1  = 9
        let sys_ftrSAR2  = 10
        let sys_ftrSAR3  = 11
        let sys_JSTARS   = 12
        let sys_ThExp1   = 13
        let sys_ThExp2   = 14
        let sys_ConUS    = 15
        let sys_CmdCont1 = 16
        let sys_CmdCont2 = 17
        let sys_LOS1     = 18
        let sys_LOS2     = 19
        let sys_BLOS1    = 20
        let sys_BLOS2    = 21

        // Characteristic indices
        let char_IFDevCost = 0
        let char_OpsCost   = 1
        let char_Perf      = 2
        let char_DevTime   = 3

        // Capability indices
        let cap_EOIR    = 0
        let cap_SAR     = 1
        let cap_Exploit = 2
        let cap_C2      = 3
        let cap_Comm    = 4

        // Define hasSystem() function
        // Returns whether the given system is specified in the chromosome
        let hasSystem i =
            // Return whether system is specified
            architecture.[i]

        // Define hasInterface() function
        // Returns whether the given interface is specified in the chromosome
        let hasInterface i j =
            // If same system
            if i = j then
                false
            else
                // Index of interface in chromosome
                let jj = if j < i then j + 1 else j
                let k = 
                    if directedInterfaces then
                        (numSystems - 1) * (i + 1) + jj
                    else
                        let m = (min i j) + 1
                        m * numSystems - m * (m - 1) / 2 + (abs(i - j)) - 1
                // Return whether interface is specified
                architecture.[k]

        // Calculate flexibility
        let mutable maxFlex = 0
        let mutable totalFlex = 0
        for i = 0 to numSystems - 1 do
            for j = 0 to numCapabilities - 1 do
                if capabilities.[i, j] then
                    maxFlex <- maxFlex + 1
                    if hasSystem i then
                        totalFlex <- totalFlex + 1
        let flex = lowerBound + (upperBound - lowerBound) * (float totalFlex) / (float maxFlex)
        
        // Return bounded results
        max lowerBound (min upperBound (flex + bias))
