% Constrain given architecture via chromosome fixing.

function fixed = FixChromosome( ...
    characteristics,capabilities,feasibleInterfaces, ...
    directedInterfaces,architecture)

% Number of elements
numSystems         = size(characteristics,1);
numCharacteristics = size(characteristics,2);
numCapabilities    = size(capabilities,2);

% System indices
sys_fighter1 = 1;
sys_fighter2 = 2;
sys_fighter3 = 3;
sys_RPA1     = 4;
sys_RPA2     = 5;
sys_RPA3     = 6;
sys_RPA4     = 7;
sys_U2       = 8;
sys_DSP      = 9;
sys_ftrSAR1  = 10;
sys_ftrSAR2  = 11;
sys_ftrSAR3  = 12;
sys_JSTARS   = 13;
sys_ThExp1   = 14;
sys_ThExp2   = 15;
sys_ConUS    = 16;
sys_CmdCont1 = 17;
sys_CmdCont2 = 18;
sys_LOS1     = 19;
sys_LOS2     = 20;
sys_BLOS1    = 21;
sys_BLOS2    = 22;

% Characteristic indices
char_IFDevCost = 1;
char_OpsCost   = 2;
char_Perf      = 3;
char_DevTime   = 4;

% Capability indices
cap_EOIR    = 1;
cap_SAR     = 2;
cap_Exploit = 3;
cap_C2      = 4;
cap_Comm    = 5;

% Define HasSystem() function
HasSystem = @(arch,i) HasSystemF(arch,i);

% Define HasInterface() function
HasInterface = @(arch,i,j) HasInterfaceF( ...
    arch,numSystems,directedInterfaces,i,j);

% Define SetSystem() function
SetSystem = @(arch,i,val) SetSystemF(arch,i,val);

% Define SetInterface() function
SetInterface = @(arch,i,j,val) SetInterfaceF( ...
    arch,numSystems,directedInterfaces,i,j,val);

% Fixed chromosome
fixed = architecture;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replace below with the actual constraints!       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is just an example of using the interfaces! %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Identify and add missing capabilities
for i = 1:numCapabilities
    j = 1;
    k = 0;
    hasCapability = false;
    while ((~hasCapability) && (j<=numSystems))
        if (capabilities(j,i))
            if (HasSystem(fixed,j))
                hasCapability = true;
            else
                k = j;
            end
        end
        j = j+1;
    end
    if ((~hasCapability) && (k>0))
        fixed = SetSystem(fixed,k,true);
    end
end

% Remove infeasible interfaces from architecture
for i = 1:numSystems
    for j = 1:numSystems
        % If different systems
        if (i ~= j)
            % If interface is specified
            if (HasInterface(fixed,i,j))
                % If infeasible, remove
                if (~(HasSystem(fixed,i) && HasSystem(fixed,j) && ...
                      feasibleInterfaces(i,j)))
                    fixed = SetInterface(fixed,i,j,false);
                end
            end
        end
    end
end
