% Calculate the objective's value for a given architecture.

function objective = Objective_Flexibility( ...
    characteristics,capabilities,feasibleInterfaces, ...
    directedInterfaces,negotiatedTime,negotiatedCost,architecture)

% Bounds
lowerBound =   0;
upperBound = 100;
bias       =   0;

% Number of elements
numSystems         = size(characteristics,1);
numCharacteristics = size(characteristics,2);
numCapabilities    = size(capabilities,2);

% System indices
sys_fighter1 = 1;
sys_fighter2 = 2;
sys_fighter3 = 3;
sys_RPA1     = 4;
sys_RPA2     = 5;
sys_RPA3     = 6;
sys_RPA4     = 7;
sys_U2       = 8;
sys_DSP      = 9;
sys_ftrSAR1  = 10;
sys_ftrSAR2  = 11;
sys_ftrSAR3  = 12;
sys_JSTARS   = 13;
sys_ThExp1   = 14;
sys_ThExp2   = 15;
sys_ConUS    = 16;
sys_CmdCont1 = 17;
sys_CmdCont2 = 18;
sys_LOS1     = 19;
sys_LOS2     = 20;
sys_BLOS1    = 21;
sys_BLOS2    = 22;

% Characteristic indices
char_IFDevCost = 1;
char_OpsCost   = 2;
char_Perf      = 3;
char_DevTime   = 4;

% Capability indices
cap_EOIR    = 1;
cap_SAR     = 2;
cap_Exploit = 3;
cap_C2      = 4;
cap_Comm    = 5;

% Define HasSystem() function
HasSystem = @(i) HasSystemF(architecture,i);

% Define HasInterface() function
HasInterface = @(i,j) HasInterfaceF( ...
    architecture,numSystems,directedInterfaces,i,j);

% Calculate flexibility
maxFlex = 0;
totalFlex = 0;
for i = 1:numSystems
    for j = 1:numCapabilities
        if (capabilities(i,j))
            maxFlex = maxFlex+1;
            if (HasSystem(i))
                totalFlex = totalFlex+1;
            end
        end
    end
end
flex = lowerBound + (upperBound - lowerBound)*totalFlex/maxFlex;

% Return bounded results
objective = max(lowerBound,min(upperBound,flex+bias));
