# Container class.
class Objective:

    # Class constructor
    def __init__(self):
        pass

    # Calculate the objective's value for a given architecture.
    def Objective_Flexibility(self, characteristics, capabilities,
                              feasibleInterfaces, directedInterfaces,
                              negotiatedTime, negotiatedCost, architecture):

        # Bounds
        lowerBound =   0.0
        upperBound = 100.0
        bias       =   0.0

        # Number of elements
        numSystems         = characteristics.GetLength(0)
        numCharacteristics = characteristics.GetLength(1)
        numCapabilities    = capabilities.GetLength(1)

        # System indices
        sys_fighter1 = 0
        sys_fighter2 = 1
        sys_fighter3 = 2
        sys_RPA1     = 3
        sys_RPA2     = 4
        sys_RPA3     = 5
        sys_RPA4     = 6
        sys_U2       = 7
        sys_DSP      = 8
        sys_ftrSAR1  = 9
        sys_ftrSAR2  = 10
        sys_ftrSAR3  = 11
        sys_JSTARS   = 12
        sys_ThExp1   = 13
        sys_ThExp2   = 14
        sys_ConUS    = 15
        sys_CmdCont1 = 16
        sys_CmdCont2 = 17
        sys_LOS1     = 18
        sys_LOS2     = 19
        sys_BLOS1    = 20
        sys_BLOS2    = 21

        # Characteristic indices
        char_IFDevCost = 0
        char_OpsCost   = 1
        char_Perf      = 2
        char_DevTime   = 3

        # Capability indices
        cap_EOIR    = 0
        cap_SAR     = 1
        cap_Exploit = 2
        cap_C2      = 3
        cap_Comm    = 4

        # Define hasSystem() function
        # Returns whether the given system is specified in the chromosome
        def hasSystem(i):
            # Return whether system is specified
            return architecture[i]

        # Define hasInterface() function
        # Returns whether the given interface is specified in the chromosome
        def hasInterface(i, j):
            # If same system
            if i == j:
                return False
            else:
                # Index of interface in chromosome
                jj = j + 1 if j < i else j
                m = min(i, j) + 1
                k = (numSystems - 1) * (i + 1) + jj if directedInterfaces else \
                    m * numSystems - m * (m - 1) / 2 + abs(i - j) - 1
                # Return whether interface is specified
                return architecture[k]

        # Calculate flexibility
        maxFlex = 0
        totalFlex = 0
        for i in range(0, numSystems):
            for j in range(0, numCapabilities):
                if capabilities[i, j]:
                    maxFlex += 1
                    if hasSystem(i):
                        totalFlex += 1
        flex = lowerBound + (upperBound - lowerBound) * totalFlex / float(maxFlex)

        # Return bounded results
        return max(lowerBound, min(upperBound, flex + bias))
